unit C_Cocktail;
{** C_Cocktail.pas
 ** v1.4 __JF 22/03/2001
 **    Ajoute methode CleanWithPath et MemoryUsed
 **}
interface
uses
  Windows, Sysutils, Classes,
  U_Cocktail, U_CocktailConst,
  U_ReplaceTexte, U_FuncDebug
  ;

type
  TProcShowInfo = procedure (const info : string; NumElem : integer) of object;

  TListCocktail = class (TStringList)
  private
   // CallBack
    FOnShowInfo : TProcShowInfo;
   // Internal
    FNbElemToLoad : integer;
    FTemplateHTML : string;
    FTemplateRTF  : string;
    function  GetRecetteByIndex(index : integer) : TCocktailRecette;
    function  GetRecettePointerByIndex(index : integer) : pTCocktailRecette;
    procedure SetRecetteByIndex(index : integer ; recette : TCocktailRecette);
    procedure SetRecettePointerByIndex(index : integer; pRecette : pTCocktailRecette);
    function  GetRecetteAsString(index : integer) : string;
    function  GetMemoryUsed : integer; // v1.4 __JF 22/03/2001
//
    procedure AddMacros(template : TReplaceMacro ; idx : integer);

   // Callback callers proc
    procedure ShowInfo(const info : string; numElem : integer);
  protected
  public
    Destructor Destroy; override;
    constructor Create;
    function  AddRecette(const Recette : pTCocktailRecette) : integer;
    function  AddRecetteAndCopy(const Recette : TCocktailRecette) : integer;
    function  DelRecetteIdx(idx : integer) : integer;
    procedure ReplaceRecette(const index : integer; newrecette : pTCocktailRecette);
    function  GetRecetteIdxByNom(const nom : string) : integer;
    procedure SaveToFile(const filename : string); OverRide;
    function  LoadFromFile(const filename : string) : integer;
    procedure ExportOneRecette(const filename : string ; idx : integer);
    procedure ExportOneRecetteAsHTML(const filename : string ; idx : integer); // 11/01/2001
    procedure ExportOneRecetteAsRTF(const filename : string ; idx : integer); // 11/01/2001
    procedure ExportAllRecette(const Filename : string);
    procedure DeleteAllRecette;
    procedure Clean;
    procedure CleanWithPath(const VideoPath, PhotoPath : string); // v1.4 __JF 22/03/2001
    property  Items[Index: Integer]: TCocktailRecette read GetRecetteByIndex write SetRecetteByIndex; default;
    property  ItemsPointer[index : integer] : pTCocktailRecette read GetRecettePointerByIndex write SetRecettePointerByIndex;
    property  OnShowInfo : TProcShowInfo read FOnShowInfo write FOnShowInfo;
    property  NbElemToLoad : integer read FNbElemToLoad;
    property  RecetteAsString[index : Integer] : string read GetRecetteAsString;
    property  TemplateHTML : string read FTemplateHTML write FTemplateHTML;
    property  TemplateRTF  : string read FTemplateRTF write FTemplateRTF;
    property  MemoryUsed : integer read GetMemoryUsed; // v1.4 __JF 22/03/2001
  end;
function FindFileInPath(const filename, path1, path2, path3 : string) : string;

implementation

uses
  u_FuncString,
  U_FuncFile,
  C_LinearMemory;

{$D-}
{^^^ desactiver le debuggage seulement pour les fonctions de bas niveau }

function  TListCocktail.GetRecetteByIndex(index : integer) : TCocktailRecette;
var
  p : pointer;
begin
  p := GetObject(index);
  result := pTCocktailRecette(p)^;
end;

function  TListCocktail.GetRecettePointerByIndex(index : integer) : pTCocktailRecette;
begin
  result := pTCocktailRecette(GetObject(index));
end;

procedure TListCocktail.SetRecettePointerByIndex(index : integer; pRecette : pTCocktailRecette);
begin
  Objects[index] := pointer(pRecette);
end;

function  TListCocktail.GetRecetteAsString(index : integer) : string;
var
  pCock : pTCocktailRecette;
begin
  try
    pCock := pTCocktailRecette(GetObject(index));
  except
    pCock := nil;
  end;
  if (pCock <> nil) then
  begin
    result := 'Nom : '+pCock^.Nom+CRLF
             +'Melangeur : '+CocktailMelangeur2Str(pCock^.Melangeur)+CRLF
             +'Verre : '+CocktailVerre2Str(pCock^.Verre)+CRLF
             +'Cout : '+CocktailCout2Str(pCock^.Cout)+CRLF
             +'Nombre de personne :'+IntToStr(pCock^.NbPersonne)+CRLF
             +'Gout : '+CocktailGout2Str(pCock^.Gout) + CRLF
             +'Alcool : '+CocktailAlcool2Str(pCock^.Alcool) + CRLF
             +'Temperature : '+CocktailTemperature2Str(pCock^.Temperature) + CRLF
             +'Ingrdients :'+CRLF+pCock^.Ingredients+CRLF
             +'Recette :'+CRLF+pCock^.Recette+CRLF
             ;
   OutputDebug(result);

  end
  else
    result := '';
end;

procedure TListCocktail.SetRecetteByIndex(index : integer ; recette : TCocktailRecette);
begin
  Objects[index] := @recette;
end;
{$D+}

procedure TListCocktail.DeleteAllRecette;
var
  pRecette : PTCocktailRecette;
  i : integer;
begin
  for i := 0 to Count - 1 do
  begin
    pRecette := PTCocktailRecette(Objects[i]);
    if (pRecette <> nil) then
    begin
      pRecette^.Photo := '';
      pRecette^.Video := '';
      pRecette^.Ingredients := '';
      pRecette^.Recette := '';
      Dispose(pRecette);
    end;
  end;
  Clear;

end;

procedure TListCocktail.ShowInfo(const info : string; numElem : integer);
begin
  if assigned (FOnShowInfo) then
    FOnShowInfo(info, numElem);
end;

// Public

Destructor TListCocktail.Destroy;
begin
  DeleteAllRecette;
  inherited;
end;

constructor TListCocktail.Create;
begin
  FnbElemToLoad := -1;
  inherited;
end;

function TListCocktail.AddRecette(const Recette : pTCocktailRecette) : integer;
begin
  result := AddObject(recette^.Nom, TObject(Recette));
end;

function TListCocktail.AddRecetteAndCopy(const Recette : TCocktailRecette) : integer;
var
  pRecette : PTCocktailRecette;
begin
  new (pRecette);
  system.Move(Recette, pRecette^, SizeOf(TCocktailRecette));
  result := AddRecette(pRecette);
end;

procedure TListCocktail.ReplaceRecette(const index : integer ; newrecette : pTCocktailRecette);
var
  pOldRecette : PTCocktailRecette;
begin
  Self.Sorted := false;
  pOldRecette := PTCocktailRecette(Objects[index]);
  Dispose(pOldRecette);
  Objects[index] := TObject(newrecette);
  Strings[index] := newRecette^.Nom;
  Self.Sort;
  Self.Sorted := true;
end;

function  TListCocktail.GetRecetteIdxByNom(const nom : string) : integer;
var
  idx : integer;
begin
  result := -1;
  for idx := 0 to Count - 1 do
  begin
    if (PTCocktailRecette(Objects[idx])^.Nom = Nom) then
    begin
      result := idx;
      break;
    end;
  end;
end;

function  TListCocktail.GetMemoryUsed : integer;
// v1.4 __JF 22/03/2001
// Retourne la taille, en octets, des donnes prsentent en mmoire.
var
  memused : integer;
  idx : integer;
begin
  memused := 0;
  for idx := 0 to count - 1 do
  begin
    memused := memused + Length(PTCocktailRecette(Objects[idx])^.Ingredients);
    memused := memused + Length(PTCocktailRecette(Objects[idx])^.Recette);
    memused := memused + Length(PTCocktailRecette(Objects[idx])^.Photo);
    memused := memused + Length(PTCocktailRecette(Objects[idx])^.Video);
  end;
  memUsed := memused + (( Count ) * SizeOf(TCocktailRecette));
  result := MemUsed;
end;

type
  TIntegerCastChar = array[0..3] of char;
  pTIntegerCastChar = ^TIntegerCastChar;
  TArrayOfChar = array[0..1024] of char;
  pTArrayOfChar = ^TArrayOfChar;
  TArrayOfMultiChar  = array[0..1024] of char;
  pTArrayOfMultiChar = ^TArrayOfMultiChar;

procedure TListCocktail.SaveToFile(const filename : string);
var
  fd : THANDLE;
  bufLen : integer;
  i : integer;
  mem : TWriteLinearMemory;
  nbElem : integer;
  pRecette : PTCocktailRecette;
  temp_photo : string;
  temp_video : string;
  temp_ingredients : string;
  temp_recette : string;
begin
  OutputDebug('Save '+filename);
  fd := CreateFile(
    PCHAR(Filename),
    GENERIC_WRITE,
    FILE_SHARE_READ,
    nil,
    CREATE_ALWAYS,
    FILE_ATTRIBUTE_NORMAL,
    0);
  if (fd <> INVALID_HANDLE_VALUE) then
  begin
    nbElem := Count;
    if (WriteFile(fd, nbElem, SizeOf(nbElem), buflen, nil) <> false) then
    begin
      for i := 0 to Count - 1 do
      begin
        pRecette := ItemsPointer[i];
        mem := TWriteLinearMemory.Create;
        temp_photo := pRecette^.Photo;
        temp_video := pRecette^.video;
        temp_Ingredients := pRecette^.Ingredients;
        temp_Recette := pRecette^.Recette;
        buflen := SizeOf(buflen)+ // 4octets entete
                  SizeOf(TCocktailRecette) + // 106 octets ?
                  Length(temp_Photo) +
                  1+ // #0
                  Length(temp_Video)+
                  1+ // #0
                  Length(temp_Ingredients)+
                  1+ // #0
                  Length(temp_Recette)+
                  1 // #0
                  ;
        mem.Size := BufLen;
        Mem.WriteInteger(BufLen);
        Mem.WriteBuf(pRecette, SIZEOF(TCocktailRecette)-16);
        mem.WriteInteger($0);
        mem.WriteInteger($0);
        mem.WriteInteger($0);
        mem.WriteInteger($0);
        mem.WriteStringAZT(temp_Photo);
        mem.WriteStringAZT(temp_video);
        mem.WriteStringAZT(temp_Ingredients);
        mem.WriteStringAZT(temp_Recette);
        if (WriteFile(fd, mem.Buf^, BufLen, BufLen, nil)<>false) then
        begin
          if (i mod 100 = 0) then
            ShowInfo(pRecette^.nom, i);
        end
        else
        begin
          ShowInfo(Format(ERR_LOAD, [SysErrorMessage(GetLastError)]), 0);
        end;
        mem.Free;
      end; // for du parcour de toutes les recettes
    end
    else
      ShowInfo(Format(ERR_LOAD, [SysErrorMessage(GetLastError)]), 0);
    CloseHandle(fd);
  end
  else
  begin
    ShowInfo(Format(ERR_LOAD, [SysErrorMessage(GetLastError)]), 0);
    raise EFCreateError.Create(SysErrorMessage(GetLastError)+' - Vrifiez que le disque n''est pas plein ou protger contre la lecture.');
  end;
  OutputDebug('End of save');
end;

function  TListCocktail.LoadFromFile(const filename : string) : integer;
var
  fd : THANDLE;
  pRecette : PTCocktailRecette;
  mem : TReadLinearMemory;
  buf : pointer;
  nbElem : integer;
  buflen : integer;
  nlu : integer;
  res : boolean;
label
  OnReadError, OnReadRecord;
begin
  mem := nil; fd := 0; buf := nil;// avoid warning
  OutputDebug('Load '+filename);
try
  nbElem := 0;
  FnbElemToLoad := 0;
  fd := CreateFile(PCHAR(Filename),
                     GENERIC_READ,
                     FILE_SHARE_READ,
                     nil,
                     OPEN_EXISTING,
                     FILE_ATTRIBUTE_NORMAL,
                     0);
  if (fd <> INVALID_HANDLE_VALUE) then
  begin
    DeleteAllRecette;
    res := ReadFile(fd, FNbElemToLoad, SizeOf(FNbElemToLoad), nlu, nil);
    if (res = false) or (nlu = 0) then goto OnReadError;
OnReadRecord:
    // Lecture de la taille du buffer
    res := ReadFile(fd, buflen, SizeOf(buflen), nlu, nil);
    if (res = false) or (nlu = 0) then goto OnReadError;
    dec(BufLen, 4);
    mem := TReadLinearMemory.Create;
    mem.Size := buflen;
    GetMem(buf, buflen);
    mem.buf := buf;
    res := ReadFile(fd, mem.Buf^, bufLen, nlu, nil);
    if (res = false) or (nlu = 0) then goto OnReadError;
    new(pRecette);
    mem.ReadBuf(pRecette, SizeOf(TCocktailRecette) - 4*4);
    mem.ReadInteger;
    mem.ReadInteger;
    mem.ReadInteger;
    mem.ReadInteger; // @ of String : ignore
    pRecette^.Photo := mem.ReadStringAZT;
    pRecette^.Video := mem.ReadStringAZT;
    pRecette^.Ingredients := mem.ReadStringAZT;
    pRecette^.Recette := mem.ReadStringAZT;
    inc(nbElem);
    pRecette^.Id := nbElem;
    AddRecette(pRecette);
    if (nbElem mod 100 = 0) then
      ShowInfo(pRecette^.Nom, nbElem);
    mem.Free; mem := nil; Freemem(buf); buf := nil;
    goto OnReadRecord;
OnReadError:
    if (res = false) then
       ShowInfo(Format(ERR_SAVE, [SysErrorMessage(GetLastError)]), 0);
  end;
  CloseHandle(fd);
  result := 1;
except
  if (mem <> nil) then mem.Free;
  if (mem <> nil) then FreeMem(buf);
  CloseHandle(fd);
  result := 0;
end;
  Outputdebug('End of load');
end;

function  IntToCroix(i : integer) : string;
var
  s : string;
begin
  s := '';
  while (i<> 0) do
  begin
    s := '+'+s;
    dec(i);
  end;
  result := s;
end;

procedure TListCocktail.ExportOneRecette(const filename : string ; idx : integer);
var
  pCock : pTCocktailRecette;
  fd : TextFile;
begin
  try
    pCock := ItemsPointer[idx];
    AssignFile(fd, filename);
    Rewrite(fd);
    Writeln(fd, format('%3d. %s', [idx+1, pCock^.Nom]));
    Writeln(fd);
    Writeln(fd, Format('     Prix   : %s', [CocktailCout2Str(pCock^.Cout)]));
    Writeln(fd, Format('     Alcool : %s',[CocktailAlcool2Str(pCock^.Alcool)]));
    Writeln(fd, Format('     Gout   : %s', [CocktailGout2str(pCock^.Gout)]));
    Writeln(fd, Format('     Temperature : %s', [CocktailTemperature2str(pCock^.Temperature)]));
    Writeln(fd);
    Writeln(fd, Format('Mlangeur : %s', [CocktailMelangeur2Str(pCock^.Melangeur)]));
    Writeln(fd, Format('Servir dans un verre : %s', [CocktailVerre2Str(pCock^.Verre)]));
    Writeln(fd);
    Writeln(fd, pCock^.Ingredients);
    Writeln(fd);
    Writeln(fd, pCock^.Recette);
    CloseFile(fd);
  except
  end;
end;

procedure TListCocktail.AddMacros(Template : TReplaceMacro ; idx : integer);
var
  pCock : pTCocktailRecette;
begin
  pCock := ItemsPointer[idx];
  Template.MacroValue.Add('%APPNAME='+FULL_APPNAME);
  Template.MacroValue.Add('%RECETTE.NOM='+pCock^.Nom);
  Template.MacroValue.Add('%RECETTE.INDEX='+IntToStr(idx+1));
  Template.MacroValue.Add('%RECETTE.MELANGEUR='+CocktailMelangeur2Str(pCock^.Melangeur));
  Template.MacroValue.Add('%RECETTE.VERRE='+CocktailVerre2Str(pCock^.Verre));
  Template.MacroValue.Add('%RECETTE.PRIX='+CocktailCout2Str(pCock^.Cout));
  Template.MacroValue.Add('%RECETTE.TEMPERATURE='+CocktailTemperature2str(pCock^.Temperature));
  Template.MacroValue.Add('%RECETTE.ALCOOL='+CocktailAlcool2Str(pCock^.Alcool));
  Template.MacroValue.Add('%RECETTE.GOUT='+CocktailGout2str(pCock^.Gout));
  Template.MacroValue.Add('%RECETTE.PHOTO='+pCock^.Photo);
  Template.MacroValue.Add('%RECETTE.MAX='+IntToStr( Self.Count ));
  Template.MacroValue.Add('%DATETIME='+DateTimeToStr(now));
end;

function ConvertCRLFtoBR(const s : string) : string;
var
  i : integer;
  r : string;     // 0d 0a
begin
  r := '';
  for i := 1 to length(s) do
  begin
    if (s[i] = #$0D) and ( i < length(s)-1) and (s[i+1] = #$0A) then
      r := r + '<br>'
    else
      r := r + s[i];
  end;
  result := r;
end;

procedure TListCocktail.ExportOneRecetteAsHTML(const filename : string ; idx : integer); // 11/01/2001
var
  Template : TReplaceMacro;
  pCock : pTCocktailRecette;
  template_file : String;
begin
  Template := TReplaceMacro.Create;
  try
    pCock := ItemsPointer[idx];
    template_file := PathToExe() + FTemplateHTML;
    Template.OriginalText.LoadFromFile(template_file);
    AddMacros(Template, idx);
    Template.MacroValue.Add('%RECETTE.INGREDIENTS='+ConvertCRLFtoBR(pCock^.Ingredients));
    Template.MacroValue.Add('%RECETTE.RECETTE='+ConvertCRLFtoBR(pCock^.Recette));
    Template.ReplaceMacro;
    Template.ModifiedText.SaveToFile(filename);
  finally
    Template.Free;
  end;
end;

function ConvertCRLFtoRTF(const s : string) : string;
var
  i : integer;
  r : string;     // 0d 0a
begin
  r := '';
  for i := 1 to length(s) do
  begin
    if (s[i] = #$0D) and ( i < length(s)-1) and (s[i+1] = #$0A) then
      r := r + '\par '
    else
      r := r + s[i];
  end;
  result := r;
end;

procedure TListCocktail.ExportOneRecetteAsRTF(const filename : string ; idx : integer); // 11/01/2001
var
  Template : TReplaceMacro;
  pCock : pTCocktailRecette;
  template_file : String;
begin
  Template := TReplaceMacro.Create;
  try
    pCock := ItemsPointer[idx];
    template_file := PathToExe() + FTemplateHTML;
    Template.OriginalText.LoadFromFile(template_file);
    AddMacros(Template, idx);
    Template.MacroValue.Add('%RECETTE.INGREDIENTS='+ConvertCRLFtoRTF(pCock^.Ingredients));
    Template.MacroValue.Add('%RECETTE.RECETTE='+ConvertCRLFtoRTF(pCock^.Recette));
    Template.ReplaceMacro;
    Template.ModifiedText.SaveToFile(filename);
  Finally
    Template.Free;
  end;
end;

procedure TListCocktail.ExportAllRecette(const Filename : string);
var
  pCock : pTCocktailRecette;
  fd : TextFile;
  i : integer;
{
  sparateur tabulation
}
begin
  try
    AssignFile(fd, filename);
    Rewrite(fd);
    Writeln(fd, 'nom'#9'prix'#9'alcool'#9'gout'#9'temperature'#9'melangeur'#9'verre'#9'"ingredients"'#9'"recette"');
    for i := 0 to Count - 1 do
    begin
      pCock := ItemsPointer[i];
      Writeln(fd,
              Format('%s'#9'%s'#9'%s'#9'%s'#9'%s'#9'%s'#9'%s'#9'"%s"'#9'"%s"',
              [pCock^.Nom,
               CocktailCout2Str(pCock^.Cout),
               CocktailAlcool2Str(pCock^.Alcool),
               CocktailGout2Str(pCock^.Gout),
               CocktailTemperature2str(pCock^.Temperature),
               CocktailMelangeur2Str(pCock^.Melangeur),
               CocktailVerre2Str(pCock^.Verre),
               RemoveCRLFfromStr(pCock^.Ingredients),
               RemoveCRLFfromStr(pCock^.Recette)
              ]));
    end;

    CloseFile(fd);
  except
  end;
end;

function  TListCocktail.DelRecetteIdx(idx : integer) : integer;
begin
  if (idx >= count) then
  begin
    idx := Count - 1;
  end;

  pTCocktailRecette(ItemsPointer[idx])^.Photo := '';
  pTCocktailRecette(ItemsPointer[idx])^.Video := '';
  pTCocktailRecette(ItemsPointer[idx])^.Ingredients := '';
  pTCocktailRecette(ItemsPointer[idx])^.Recette := '';
  FreeMem(pTCocktailRecette(ItemsPointer[idx]));
  ItemsPointer[idx] := nil;
  Delete(idx);
  result := Count;
end;

procedure TListCocktail.Clean;
var
  i : integer;
begin
  for i := 0 to Count - 1 do
  begin
    pTCocktailRecette(ItemsPointer[i])^.Nom := Trim(pTCocktailRecette(ItemsPointer[i])^.Nom);
    pTCocktailRecette(ItemsPointer[i])^.Auteur := Trim(pTCocktailRecette(ItemsPointer[i])^.Auteur);
    pTCocktailRecette(ItemsPointer[i])^.Photo := Trim(pTCocktailRecette(ItemsPointer[i])^.Photo);
    pTCocktailRecette(ItemsPointer[i])^.Video := Trim(pTCocktailRecette(ItemsPointer[i])^.Video);
    pTCocktailRecette(ItemsPointer[i])^.Ingredients := CleanStrings(pTCocktailRecette(ItemsPointer[i])^.Ingredients);
    pTCocktailRecette(ItemsPointer[i])^.Recette := CleanStrings(pTCocktailRecette(ItemsPointer[i])^.Recette);
  end;
end;

procedure TListCocktail.CleanWithPath(const VideoPath, PhotoPath : string);
// v1.4 __JF 22/03/2001
// Nettoyage des recettes, avec en plus la simplification du chemin
// des photos/vidos. Les chemins doivent se terminer par '\'
var
  i : integer;
  newfilename, filename,tmp : string;
begin
  for i := 0 to Count - 1 do
  begin
    pTCocktailRecette(ItemsPointer[i])^.Nom := Trim(pTCocktailRecette(ItemsPointer[i])^.Nom);
    pTCocktailRecette(ItemsPointer[i])^.Auteur := Trim(pTCocktailRecette(ItemsPointer[i])^.Auteur);
    filename := pTCocktailRecette(ItemsPointer[i])^.Photo;
    if (filename <> '') then
    begin
      tmp := ExtractFileName(filename);
      newfilename := FindFileInPath(tmp, photopath, PathToExe+photopath, PathToExe);
      if (newfilename = '') then
        OutputDebugFormat('!! Recette %d : photo %s n''existe pas', [i, newfilename])
      else
        if (newfilename <> filename) then
        begin
          OutputDebugFormat('Recette %d : %s chang en %s', [i, filename, newfilename])
        end;
      pTCocktailRecette(ItemsPointer[i])^.Photo := newfilename;
    end;
    // On fait pareil pour la video
    {
    filename := pTCocktailRecette(ItemsPointer[i])^.Video;
    // A faire : la video
    }
    pTCocktailRecette(ItemsPointer[i])^.Video := '';
    pTCocktailRecette(ItemsPointer[i])^.Ingredients := CleanStrings(pTCocktailRecette(ItemsPointer[i])^.Ingredients);
    pTCocktailRecette(ItemsPointer[i])^.Recette := CleanStrings(pTCocktailRecette(ItemsPointer[i])^.Recette);
  end;
end;

function FindFileInPath(const filename, path1, path2, path3 : string) : string;
var
  fname : string;
  trythis : string;
begin
  if (FileExists(FileName)) then
  begin
    result := FileName;
  end
  else
  begin
    fname := ExtractFileName(Filename);
    trythis := path1+fname;
    if (FileExists(trythis)) then
    begin
      result := trythis;
    end
    else
    begin
      trythis := path2+'\'+fname;
      if (FileExists(trythis)) then
      begin
        result := trythis;
      end
      else
      begin
        trythis := path3+'\'+fname;
        if (FileExists(trythis)) then
        begin
          result := trythis;
        end
        else
        begin
          result := '';
        end;
      end;
    end;
  end;
end;


end.
