unit U_FuncFile;
{** u_FuncFile.pas
 ** __JF 22/03/2001
 ** Unit for manipulating file functions
 **}

interface

uses
  windows;

function GetINIFileNameFromExe(const exe : string) : string;
// return a filename = exe without ext + '.ini'

function GetINIFileName(const ini : string) : string;
// Return an ini filename with full path

function PathToExe : string;
// Return the path to the exe

function GetFilenameExtension(filename : string) : string;
// return the extension (BMP) of a filename (prout.bmp)

function ExtractFileNameWithoutExt(FileNames:string):string;
// return the filename without the extension (prout.bmp -> prout)

function FileNameToValidFileName(const filename : string) : string;
// Replace invalid char of filename by a '_'

function GetFileSizeAsString(const filename : string) : string;
// Return the size of the file, as string

function GetFileSizeInKB(const FileName : string) : integer;
// Return the size, in KB of a file, as string

procedure AppendSlashToPath(var path : string);
// add a trailing / to the path (if there is not a / already)

function SimplifyPath(var path : string) : integer;
// simplify a path from 'c:\temp\..\winnt' to 'c:\winnt'

function AbsolutePathToRelative(const filename, basepath : string) : string;
//

function  GetShellInfo(filename : string ; pHIcon : pointer ) : string;
// Return the shell description of a file. if PHIcon is a pointer to HICON, the HICON is returned

implementation

uses
  ShellAPI,
  sysutils;

function GetINIFileNameFromExe(const exe : string) : string;
// return a filename = exe without ext + '.ini'
var
  i : integer;
  f : string;
begin
  result := '';
  if (exe = '') then exit;
  f := exe;
  for i := length(f) downto 1 do
  begin
    if (f[i] = '.') then // search for the extension
    begin
      result := copy(f, 1, i)+'.ini';
      exit;
    end;
  end;
  result := '';
end;

function GetINIFileName(const ini : string) : string;
// Return an ini filename with full path
begin
  result := ExtractFilePath(ParamStr(0))+ini;
end;

function GetFilenameExtension(filename : string) : string; // in 'prout.bmp' out 'BMP'
var
  ext : string;
begin
  ext := ExtractFileExt(filename);
  if (ext <> '') then
    ext := AnsiUpperCase(copy(ext, 2, length(ext)));
  result := ext;
end;

function FileNameToValidFileName(const filename : string) : string;
const
  INVALIDCHAR = '/\:*?"<>|';
  VALIDCHAR='_';
var
  i : integer;
  s : string;
begin
  s := '';
  for i := 1 to length(FileName) do
  begin
    if (pos(FileName[i], INVALIDCHAR) <> 0) then
    begin
      s := s + VALIDCHAR;
    end
    else
      s := s + FileName[i];
  end;
  result := s;
end;

function GetFileSizeAsString(const filename : string) : string;
var
  lo : DWORD;
  hi : DWORD;
  fd : THANDLE;
  l : TLargeInteger;
begin
  fd := CreateFile(PCHAR(filename),
                   GENERIC_READ,
                   FILE_SHARE_READ,
                   nil,
                   OPEN_EXISTING,
                   FILE_ATTRIBUTE_NORMAL,
                   0);
  if (fd = INVALID_HANDLE_VALUE) then
    result := '???'
  else
  begin
    lo := GetFileSize(fd, @hi);
    if (lo = -1) and (GetLastError <> NO_ERROR) then
    begin
      result := '???';
    end
    else
    begin
      l.LowPart := lo;
      l.HighPart := hi;
      if (l.QuadPart <= 1024) then
        result := IntToStr(l.LowPart)
      else if (l.QuadPart <= (1024 * 1024)) then
        result := IntToStr(l.LowPart div 1024)+'K'
      else if (l.QuadPart <= (1024 * 1024*1024)) then
        result := IntToStr(l.LowPart div (1024))+'K'
      else result := IntToStr(l.LowPart div (1024*1024))+'M';
    end;
    CloseHandle(fd);
  end;
end;

function GetFileSizeInKB(const FileName : string) : integer;
var
  lo : DWORD;
  hi : DWORD;
  fd : THANDLE;
  l : TLargeInteger;
begin
  fd := CreateFile(PCHAR(filename),
                   GENERIC_READ,
                   FILE_SHARE_READ,
                   nil,
                   OPEN_EXISTING,
                   FILE_ATTRIBUTE_NORMAL,
                   0);
  if (fd = INVALID_HANDLE_VALUE) then
    result := 0
  else
  begin
    lo := GetFileSize(fd, @hi);
    if (lo = -1) and (GetLastError <> NO_ERROR) then
    begin
      result := 0;
    end
    else
    begin
      l.LowPart := lo;
      l.HighPart := hi;
      result := Integer(l.LowPart div 1024);
    end;
    CloseHandle(fd);
  end;
end;

procedure AppendSlashToPath(var path : string);
const
  SLASH = '\';
var
  l : integer;
begin
  l := length(path);
  if (l > 0) then
  begin
    if path[l] <> SLASH then
      path := path + SLASH;
  end;
end;

function ExtractFileNameWithoutExt(FileNames:string):string;
// in : toto.txt, out : toto // !! A amlior
var
  S : string;
begin
  S:='';
  while Pos('.', FileNames) > 0 do
  begin
    S:=S+Copy(FileNames,1,Pos('.',FileNames)-1);
    Delete(FileNames,1,Pos('.',FileNames));
  end;
  result:=S;
end;

function SimplifyPath(var path : string) : integer; // c:\temp\prout\..\ -> c:\temp\
var
  lpFileName : pchar;
  lpBuffer : array[0..MAX_PATH] of char;
begin
  lpFileName := PCHAR(path);
  result := GetFullPathName(lpFileName, MAX_PATH, lpBuffer, lpFileName);
  path := lpBuffer;
end;

function  GetShellInfo(filename : string ; pHIcon : pointer ) : string;
var
  fi : TSHFileInfo;
begin
  if (SHGetFileInfo(PCHAR(filename),
                0,
                fi,
                SizeOf(TSHFileInfo),
                SHGFI_ICON or SHGFI_TYPENAME ) <> 0) then
  begin
    if (pHIcon <> nil) then
      HICON(pHIcon^) := fi.hIcon;
    result := fi.szTypeName;
  end
  else
  begin
    if (pHIcon <> nil) then
      HICON(pHIcon^) := 0;
    result := '';
  end;
end;

function PathToExe : string;
begin
  result := ExtractFilePath(paramstr(0));
end;

function AbsolutePathToRelative(const filename, basepath : string) : string;
var
  s : string;
  i : integer;
  rel : string;
  basepath2 : string;
  fn2 : string;
  drvbp : string;
  drvfn : string;
begin
  drvbp := ExtractFileDrive(UpperCase(Basepath));
  drvfn := ExtractFileDrive(UpperCase(filename));
  if (drvbp <> drvfn) then
    result := filename
  else
  begin
    if (drvbp <> '') then
      BasePath2 := Copy(BasePath, length(drvbp)+1, length(BasePath))
    else
      BasePath2 := BasePath;
    if (drvfn <> '') then
      fn2 := Copy(FileName, length(drvfn)+1, length(Filename))
    else
      fn2 := Filename;
  // BasePath2 contient le chemin, sans le lecteur
  // fn2 contient le chemin sans le lecteur aussi
    AppendSlashToPath(BasePath2);
    s := '';
    for i := 1 to length(BasePath2) do
    begin
      if (basepath[i] = '\') then
        s := s + '..\';
    end;
    rel := drvbp+BasePath2 + s + fn2;
    SimplifyPath(rel);
    result := rel;
  end;
end;

end.

