unit U_FuncString;
{** u_FuncFile.pas
 ** __JF 22/03/2001
 ** Unit for manipulating strings
 **}

interface

uses
  Classes,
  Windows;

function IsStrInStr(const substr, str : string) : boolean;
// Return if a substring is in string

function GetNumberOfStrInStr(substr, str : string) : integer;
// Return the number of substring in string

function GetNumberOfMultiString(substrs : TStrings; const str : string) : integer;
// Return the number of substring (in TStrings) in str

function BooleanToStrYN(b : boolean) : string; // Return 'Oui' or 'Non'
// Return the boolean as a string

function RemoveAccent(const str : string) : string;
// Return a string, where accent where replaced by a regular character

procedure RemoveAccentEx(var str : string);
// Modify a string, where accents are replaced by a regular character

function  CleanStrings(str : string) : string;
// str is a complex string (with CRLF). It will trim the lines.

function RemoveCRLFfromStr(const str : string ) : string;
// replace CRLF by a ';'

function InfoPourcentage(val , max : integer) : string;
// return the ratio as a percent

function Pluriel(n : integer) : string;
// return 's' if n > 1

implementation

uses
  SysUtils;

function IsStrInStr(const substr, str : string) : boolean;
begin
  result := pos(AnsiUpperCase(substr), AnsiUpperCase(str)) > 0;
end;

function GetNumberOfStrInStr(substr, str : string) : integer; // Retourne le nombre de souschaine dans une chaine
var
  r : integer;
  i,j : integer;
  find : boolean;
begin
// !! TO DO : should return the number of substring 'substr' in string 'str'
  if (substr = '') then
  begin
    result := 0;
    exit;
  end;
  substr := AnsiUpperCase(substr);
  RemoveAccentEx(SubStr);
  str := AnsiUpperCase(str);
  RemoveAccentEx(Str);
 { else
    if (IsStrInStr(substr, str) = true) then
      result := 1
    else
      result := 0;
 }
  r := 0;
  i := 1;
  while (i<length(str)) do
  begin
    if (str[i] = substr[1]) then
    begin
      find := true;
      for j := 2 to length(substr) do
      begin
        if (substr[j] <> str[i+j-1]) then
        begin
          find := false;
          break;
        end;
      end;
      if (find = true) then
      begin
        inc(r);
        i := i + length(substr)-1;
      end;
    end;
    inc(i);
  end;
  result := r;

end;

function GetNumberOfMultiString(substrs : TStrings; const str : string) : integer;
var
  i : integer;
  nbMatch : integer;
  tmpstr : TStringList;
  j : integer;
  s1,s2 : string;
begin
{  nbMatch := 0;
  for i := 0 to SubStrs.Count - 1 do
  begin
    inc(nbMatch, GetNumberOfStrInStr(SubStrs[i], str));
  end;
  result := nbMatch;}
  nbMatch := 0;
  tmpstr := TStringList.Create;
  TmpStr.Text := AnsiUpperCase(str);
  for i := 0 to SubStrs.Count - 1 do
  begin
    s1 := AnsiUpperCase(SubStrs[i]);
    RemoveAccentEx(s1);
    for j := 0 to TmpStr.Count - 1 do
    begin
      s2 := TmpStr[j];
      RemoveAccentEx(s2);
      if (pos(s1, s2) <> 0) then
      begin
        inc(nbMatch);
        break;
      end;
    end;
  end;
  TmpStr.Free;
  result := nbMatch;
end;

function BooleanToStrYN(b : boolean) : string; // Return 'Oui' or 'Non'
const
  STR_YES = 'Oui';
  STR_NO  = 'Non';
begin
  if (b) then
    result := STR_YES
  else
    result := STR_NO;
end;

const
    Accents = '';
  NoAccents = 'AAAAAACEEEEIIIIDNOOOOOOUUUUYBaaaaaaceeeeiiiionoooooouuuuyby';
function RemoveAccent(const str : string) : string;
var
  i : integer;
  p : integer;
  s : string;
begin
  s := '';
  for i := 1 to length(str) do
  begin
    p := pos(str[i], Accents);
    if (p <> 0) then
      s := s + NoAccents[p]
    else
      s := s + str[i];
  end;
  result := s;
end;

procedure RemoveAccentEx(var str : string);
var
  i : integer;
  p : integer;
begin
  for i := 1 to length(str) do
  begin
    p := pos(str[i], Accents);
    if (p <> 0) then
      str[i] := NoAccents[p];
  end;
end;

function  CleanStrings(str : string) : string;
var
  j : integer;
  templist : TStringList;
begin
    templist := TstringList.Create;
      templist.Text := str;
      for j := 0 to templist.Count - 1 do
      begin
        templist[j] := Trim(templist[j]);
      end;
      j := 0;
      while (j <= templist.Count - 1) do
      begin
        if (templist[j] = '') then
          templist.Delete(j);
        inc(j);
      end;
      result := templist.Text;
    templist.Free;
end;

function RemoveCRLFfromStr(const str : string ) : string;
var
  s : string;
  i : integer;
begin
  s := '';
  for i := 1 to length(str) do
  begin
    if (str[i] = #$0A) then
      s := s + ';'
    else
    if (str[i] <> #$0D) then
      s := s + str[i];
{
!!  ToDo : remplacer les CRLF par des CR, mais le pb c'est que
         excel ne reformate pas correctement les lignes avec des CR
    c := str[i];
    if (c = #$0D) then
      break;
    if (c = #$0A) and (i > 2) and (str[i-1] = #$0D) then
    begin // une suite $0D $0A
      c := ';';
    end;
    s := s + c;}
  end;
  result := s;
end;

function InfoPourcentage(val , max : integer) : string;
begin
  if (max = 0) then result := '?%'
  else
    result := IntToStr(trunc((val / max)*100))+'%';
end;

function Pluriel(n : integer) : string; // retourne 's' si n > 1
begin
  if (n > 1) then result := 's' else result := '';
end;

end.
