unit U_CgiFunc;

interface
const
  CRLF = #10;
  HTML_HEADER='Content-Type: text/html'+CRLF+CRLF;
  BR = '<br>';
  HR = '<hr>';
function MatchPattern(InpStr,Pattern :PChar) :Boolean;
function MatchPatternString(InpStr,Pattern : string) :Boolean;
function  ConvertCRLFtoBR(const s : string) : string;
implementation
uses
  Windows;

{*****************************************************************}
{* This function implements a subset of regular expression based *}
{* search and is based on the translation of PattenMatch() API   *}
{* of common.c in MSDN Samples\VC98\sdk\sdktools\tlist           *}
{* http://www.delphi3000.com/articles/article_1561.asp           *} 
{*****************************************************************} 
{* MetaChars are  :                                              *} 
{*            '*' : Zero or more chars.                          *} 
{*            '?' : Any one char.                                *} 
{*         [adgj] : Individual chars (inclusion).                *} 
{*        [^adgj] : Individual chars (exclusion).                *} 
{*          [a-d] : Range (inclusion).                           *} 
{*         [^a-d] : Range (exclusion).                           *} 
{*       [a-dg-j] : Multiple ranges (inclusion).                 *} 
{*      [^a-dg-j] : Multiple ranges (exclusion).                 *} 
{*  [ad-fhjnv-xz] : Mix of range & individual chars (inclusion). *} 
{* [^ad-fhjnv-xz] : Mix of range & individual chars (exclusion). *} 
{*****************************************************************} 
function MatchPatternString(InpStr,Pattern : string) :Boolean;
begin
  result := MatchPattern(PCHAR(inpstr), PCHAR(pattern));
end;

function MatchPattern(InpStr,Pattern :PChar) :Boolean;
begin
  while(True) do
  begin
    case Pattern[0] of 
      #0 :begin //End of pattern reached. 
            Result := (InpStr[0] = #0); //TRUE if end of InpStr. 
            Exit; 
          end; 

      '*':begin //Match zero or more occurances of any char. 
            if(Pattern[1] = #0)then 
            begin //Match any number of trailing chars. 
              Result := True; 
              Exit; 
            end else Inc(Pattern); 

            while(InpStr[0] <> #0)do 
            begin //Try to match any substring of InpStr. 
              if(MatchPattern(InpStr,Pattern))then 
              begin 
                Result := True; 
                Exit; 
              end; 

              //Continue testing next char... 
              Inc(InpStr); 
            end; 
          end; 

      '?':begin //Match any one char. 
            if(InpStr[0] = #0)then 
            begin 
              Result := False; 
              Exit; 
            end; 

            //Continue testing next char... 
            Inc(InpStr); 
            Inc(Pattern); 
          end; 

      '[':begin //Match given set of chars. 
            if(Pattern[1] in [#0,'[',']']) then 
            begin //Invalid Set - So no match. 
              Result := False; 
              Exit; 
            end; 

            if(Pattern[1] = '^')then 
            begin //Match for exclusion of given set... 
              Inc(Pattern,2); 
              Result := True; 
              while(Pattern[0] <> ']')do 
              begin 
                if(Pattern[1] = '-')then 
                begin //Match char exclusion range. 
                  if(InpStr[0] >= Pattern[0])and(InpStr[0] <= Pattern[2])then 
                  begin //Given char failed set exclusion range. 
                    Result := False; 
                    Break; 
                  end else Inc(Pattern,3); 
                end else 
                begin //Match individual char exclusion. 
                  if(InpStr[0] = Pattern[0])then 
                  begin //Given char failed set element exclusion. 
                    Result := False; 
                    Break; 
                  end else Inc(Pattern); 
                end; 
              end; 
            end else 
            begin //Match for inclusion of given set... 
              Inc(Pattern); 
              Result := False; 
              while(Pattern[0] <> ']')do 
              begin 
                if(Pattern[1] = '-')then 
                begin //Match char inclusion range. 
                  if(InpStr[0] >= Pattern[0])and(InpStr[0] <= Pattern[2])then 
                  begin //Given char matched set range inclusion. Continue testing... 
                    Result := True; 
                    Break; 
                  end else Inc(Pattern,3); 
                end else 
                begin //Match individual char inclusion. 
                  if(InpStr[0] = Pattern[0])then 
                  begin //Given char matched set element inclusion. Continue testing... 
                    Result := True; 
                    Break; 
                  end else Inc(Pattern); 
                end; 
              end; 
            end; 

            if(Result)then 
            begin //Match was found. Continue further. 
              Inc(InpStr); 

              //Position Pattern to char after "]" 
              while(Pattern[0] <> ']')and(Pattern[0] <> #0)do Inc(Pattern); 

              if(Pattern[0] = #0)then 
              begin //Invalid Pattern - missing "]" 
                  Result := False; 
                  Exit; 
              end else Inc(Pattern); 
            end else Exit; 
          end; 

     else begin //Match given single char. 
            if(InpStr[0] <> Pattern[0])then 
            begin 
              Result := False; 
              Break; 
            end; 

            //Continue testing next char... 
            Inc(InpStr); 
            Inc(Pattern); 
          end; 
    end; 
  end; 
end; 

(*
function  MatchPattern(s : PCHAR; pattern : PCHAR) : boolean;
{* simple pattern matching 
* metachar are :'*', '?', '[x-y]' (range), '[wxyz]'
* translated from MatchPattern() of common.c in MSDN Samples\VC98\sdk\sdktools\tlist 
*} 
var 
  c : char; 
  p : char; 
  l : char; 
begin 
  while (true) do 
  begin 
    p := Pattern^; 
    inc(Pattern); 
    case p of 
      #0 : begin // End of pattern 
             result := s^ = #0; // if end of string TRUE 
             exit;
           end;
      '*': begin 
             while(bool(s^)) do
             begin // Match zero or more char
               inc(s);
               if(MatchPattern(s, pattern)) then 
               begin 
                 result := true;
                 exit; 
               end; 
             end;
             result := MatchPattern(s, pattern); 
             exit; 
           end; 
      '?': begin 
             if (s^ = #0) then 
             begin // match any one char 
               result := false; // not end of string 
               exit; 
             end; 
             inc(s); 
           end; 
      '[' : begin 
              c := s^; // c = *s++ 
              inc(s); // match char set [0123] or [0-3] 
              if (c = #0) then 
              begin 
                result := false;
                exit; 
              end;
              l := #0; 
              p := pattern^; 
              inc(pattern); 
              while (bool(p)) do
              begin 
                if (p = ']') then 
                begin // if end of char set, then no match found
                  result := false; 
                  exit; 
                end; 
                if (p = '-') then 
                begin // check a ronge of char ? 
                  p := pattern^; 
                  if (p = #0) or (p = ']') then 
                  begin // Get high limit of range 
                    result := false; 
                    exit; 
                  end; 
                  if (c >= l) and (c <= p) then 
                  begin 
                    break; // if in range, move on (redo while) 
                  end; 
                end;
                l := p; 
                if (c = p) then
                  break; // if char matches this element, move on (redo while)
                p := pattern^; 
                inc(pattern); 
              end; 
              while (bool(p) and (p <> ']')) do
              begin // got a match in char set 
                p := pattern^; 
                inc(pattern); // skip to end of set
              end; 
            end; 
      else 
           begin 
             c := s^; 
             inc(s); 
             if (c <> p) then 
             begin // check for exact char 
               result := false; 
               exit; 
             end; 
           end; 
    end; 
  end; 
end;
  *)
function ConvertCRLFtoBR(const s : string) : string;
var
  i : integer;
  r : string;     // 0d 0a
begin
  r := '';
  for i := 1 to length(s) do
  begin
    if (s[i] = #$0D) and ( i < length(s)-1) and (s[i+1] = #$0A) then
      r := r + '<br>'
    else
      r := r + s[i];
  end;
  result := r;
end;

end.

